' === FRAMdataLogger.bas ==========================================

' Program monitors real-time temperature via a thermocouple sensor,
' shows the temperature in degrees Fahrenheit on a four-digit LED,
' accesses a FRAM storage device to store the time in minutes, and
' the temperature in degrees F, at one minute intervals.

' === Directives =======
  #com 3                         'specify serial port
  #picaxe 08M2                   'specify processor
  #terminal off                  'disable terminal window
  #no_data                       'save time downloading
  
' === Constants ========
  symbol    blank = 15           '7219 requires 15 to blank a digit
  symbol      EOF = 255          'end of file marker
  symbol     FRAM = 0xA0         'default i2c address of FRAM
  symbol     LED4 = C.0          '4-digit, 7-segment LED on pin C.0
  symbol    therm = C.4          'thermocouple on pin C.4
  
' === Variables ========   
  symbol    index = b0           'used in for/next loop
  symbol  FRAMloc = w1           'FRAM storage location
  symbol FRAMtime = w2           'used to time one minute
    
  symbol   ADCval = w3           'raw ADC value from thermocouple
  symbol    mVolt = w4           'mV level of thermo reading
  symbol    tempC = w5           'value is in degrees C
  symbol    tempF = w6           'value is in degrees F
  symbol   tempFL = b12          'hi byte of tempF variable
  symbol   tempFH = b13          'lo byte of tempF variable
  symbol  tempAvg = w7

  symbol nextTime = w8           'used for timing of main loop
  symbol   minute = w9           'program minute counter
  symbol  minuteL = b18          'hi byte of minute variable
  symbol  minuteH = b19          'lo byte of minute variable
  
' === Begin Main Program ==========================================
  adcconfig %011                  'Vref- = gnd, Vref+ = FVR
  setfreq m16
  hi2csetup i2cmaster,FRAM,i2cfast_16,i2cword

  wait 2
  time = 0
  
do
  'establish main loop iteration time of 2 seconds             '[1]
  nextTime = time + 2
  
  tempAvg = 0
  for index = 1 to 10                                          '[2]
    gosub getTempC
    tempAvg = tempAvg + tempC
    pause 400                     '100mS between temp readings
  next index
  tempAvg = tempAvg / 10
  
  gosub getTempF
  gosub displayTempF

  do
    pause 100                     'allow remainder of 2 seconds
  loop until time = nextTime      'to elapse                   '[3]
  
  FRAMtime = time // 60           'see if 1 minute has passed  '[4]
  if FRAMtime = 0 then            'if so, then
     gosub storeData              'store the data
     inc minute                   'increment minute timer
     time = 0                     'and reset the seonds counter
  endif
loop

' === End Main Program - Subroutines Follow =======================
getTempC: 'get the tempC in tenths of a degree
  fvrsetup FVR4096                'FVR = 4096mV
  readadc10 therm, ADCval         'get raw ADC value
  mVolt = 4 * ADCval              'convert ADCval to mV
  tempC = 2 * mVolt - 2500        'get temp C in tenths
return

getTempF: 'get the temperature in degrees F
  tempF = 9 * tempC / 5 + 320     'get temp F in tenths
  tempF = tempF + 5               'add 5 tenths, so we can...  '[5]
  tempF = tempF / 10              'round to the nearest degree
return

displayTempF: 'display temperature in degrees F
  serout LED4,N4800_16,(blank)    'blank the left-most digit   '[6]  
  if tempF < 100 then                                          '[7]
    serout LED4,N4800_16,(blank,#tempF)
  else
    serout LED4,N4800_16,(#tempF)
  endif
return
  
storeData: 'store time in minutes, and temperature in degrees F
  hi2cout FRAMloc, (minuteL,minuteH,tempFL,tempFH,EOF,EOF)     '[8]
  FRAMloc = FRAMloc + 4           'set up for next time
return