' === ThermoTest.bas ============================================

' Program uses a thermocouple to measure real-time temperature in
' tenths of a degree Centigrade. It then converts it to tenths of
' a degree Fahrenheit, and sends the data to the terminal window.
  
' === Directives ====
  #com 3                          'specify serial port
  #picaxe 08M2                    'specify processor
  #terminal 4800                  'open the terminal window
  #no_data                        'save time downloading
  
' === Constants =====
  symbol thermo = C.4             'thermocouple on pin C.4
  
' === Variables =====
' First 5 variables are reserved for use with bintoascii command.
  symbol     d0 = b0
  symbol     d1 = b1
  symbol     d2 = b2
  symbol     d3 = b3
  symbol     d4 = b4

  symbol ADCval = w3              'the ADC input value
  symbol  mVolt = w4              'input value in mV
  symbol  tCx10 = w5              'tempC in tenths
  symbol  tFx10 = w6              'tempF in tenths

' === Begin Main ================================================
adcconfig %011                    'Vref- = gnd, Vref+ = FVR

do
  fvrsetup FVR4096                'FVR = 4096mV
  readadc10 thermo, ADCval        'get ADCval from thermo
  mVolt = 4 * ADCval              'convert it to mV
  tCx10 = 2 * mVolt - 2500        'convert mV to tempC in tenths
  tFx10 = 9 * tCx10 / 5 + 320     'convert to tempF in tenths
  
  bintoascii tFx10,d4,d3,d2,d1,d0 'separate each digit in tFx10
  
  'send data to the terminal, with decimal point correctly placed
  sertxd ("Temp = ",d3,d2,d1,".",d0," F",CR,LF)
  wait 2                          'slow it down a bit
loop